#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: ssh-keygen_user.cgi,v 1.1.1.1 2001/12/11 06:50:38 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut


  ###  Import our library routines.
  require 'userpass-lib.pl';
  &ReadParse();

  ###  Setup local variables.
  my ($username, $keysize, $comment, $passphrase1, $passphrase2);
  my ($keygen, $newkey, $AUTHORIZED_KEYS, $dot_ssh, $UID, $GID);

  ###  Assign from our input.
  $keygen           =  "/usr/bin/ssh-keygen";
  $username         =  $ENV{'REMOTE_USER'};
  $keysize          =  1024;
  $filename         =  $in{'Filename'};
  $comment          =  $in{'Comment'};
  $passphrase1      =  $in{'Passphrase1'};
  $passphrase2      =  $in{'Passphrase2'};

  if ($ENV{'REMOTE_USER'} eq "root") {
    $AUTHORIZED_KEYS  =  "/root/.ssh/authorized_keys";
    $dot_ssh          =  "/root/.ssh";
  }
  else {
    $AUTHORIZED_KEYS  =  "/home/$username/.ssh/authorized_keys";
    $dot_ssh          =  "/home/$username/.ssh";
  }


  ###
  ###  Input validation and error checking.
  ###
  if (!$username) {
    &PrintError("Please enter an username."); exit;
  }
  elsif (!$filename) {
    &PrintError("Please enter a filename."); exit;
  }
  elsif ($filename !~ m/^[A-Za-z0-9\-\_]+$/) {
    &PrintError("Your filename must be alphanumeric."); exit;
  }
  elsif (getpwnam $username < 0) {
    &PrintError("$username does not exist."); exit;
  }
  elsif (!$comment) {
    &PrintError("Please enter a comment to identify this key pair."); exit;
  }
  elsif (!$passphrase1 || !passphrase2) {
    &PrintError("Please enter a passphrase."); exit;
  }
  elsif ($passphrase1 ne $passphrase2) {
    &PrintError("The two passphrases do not match.  Please re-enter."); exit;
  }


  ###  Save our current ID's and get our target ID's.
  my ($UID, $GID, $s_UID, $s_EUID, $s_GID, $s_EGID);

  $s_UID  = $<; $s_EUID = $>; $UID = undef;
  $s_GID  = $(; $s_EGID = $); $GID = undef;

  setpwent();
    while (my @u = getpwent()) {
      if ($u[0] eq $username) { $UID = $u[2]; $GID = $u[3]; }
    }
  endpwent();


  ###  Throw an error if we don't have $UID and $GID.
  if ( (! defined($UID)) || (! defined($GID)) ) {
    &PrintError("Unable to find your UID and GID. Contact your administrator.");
    exit;
  }


  ###
  ###  Drop privileges.
  ###
  $< = $UID;        $( = $GID;
  $) = "$GID $GID"; $> = $UID;


  ###  If $HOME/.ssh or $HOME/.ssh/authorized_keys DNE, create them.
  if ( (! -d $dot_ssh) || (! -f $AUTHORIZED_KEYS) )
  {
    umask 0077;
    mkdir $dot_ssh, 0700;
    open AK, ">$AUTHORIZED_KEYS"; close AK;
  }

  ###  Check to see if we already have keys by this name.
  if ( ( -f "$dot_ssh/$filename" ) || ( -f "$dot_ssh/$filename.pub" ) )
  {
    &PrintError("Keys by this name already exist.  Please choose another filename.");
    exit;
  }


  ###  Generate the keys then...
  $passphrase1  =~  s/\"/\\\"/g;
  $comment      =~  s/\"/\\\"/g;

  system "$keygen -b $keysize -f \"$dot_ssh/$filename\" -N \"$passphrase1\" -C \"$comment\" > /dev/null 2>&1";

  ###  ...place our new public in our authorized_keys and...
  open NEWPUB, "$dot_ssh/$filename.pub"; open AK, ">>$AUTHORIZED_KEYS";
    $newkey = <NEWPUB>; print AK $newkey;
  close AK; close NEWPUB;

  ###  ...sanitizie the ownerships/permissions.
  chmod 0700, $dot_ssh;
  chmod 0600, $AUTHORIZED_KEYS;
  chmod 0400, "$dot_ssh/$filename", "$dot_ssh/$filename.pub";


  ###
  ###  Restore privileges.
  ###
  $> = $s_EUID; $) = $s_EGID;
  $( = $s_GID;  $< = $s_UID;


  ###  Print the standard header.
  &header("Key Generation Complete", 0);


  ###  Now print the HTML so they can download it.
  print <<EOF01;
<FORM ACTION="do_download_key.cgi" METHOD="POST">
<CENTER><TABLE WIDTH="620" CELLPADDING=0 CELLSPACING=0 BORDER=0>
  <TR $tb> <TD COLSPAN=2><DIV CLASS=ttitle><B>SSH Key Generation Complete</TD></TR>

  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>

  <TR $cb>
    <TD ALIGN="CENTER"><DIV CLASS=nbody>
    <INPUT TYPE="HIDDEN" NAME="filename" VALUE="$filename">
    <INPUT TYPE="SUBMIT" VALUE="Click Here to Download Private Key">
      <BR><B>(Save this to your computer and name it something you will 
             remember.)
    </TD>
  </TR>

  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
  <TR $cb> <TD COLSPAN=2>&nbsp;</TD> </TR>
</TABLE></CENTER></FORM><P>
EOF01


  ###  Print the standard footer.
  &ryanFooter("keyring.cgi");
